<?php
namespace OrcamentoOperacional\Logic;


use \Fiscal\Entities\Notafiscal;
use \Geral\Entities\Prodfili;
use \Geral\Util\Ambiente;

/**
 *
 */
class LiberacaoOrcamento
{

    private $notaFiscal;

    /**
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public function __construct(Notafiscal $notaFiscal)
    {
        $this->notaFiscal = $notaFiscal;
    }

    /**
     * Verifica se a filial utiliza orcamento GOP.
     *
     * @static
     * @param \Geral\Entities\Filial $filial
     * @return bool
     */
    public static function filialUtilizaOrcamentoGOP(\Geral\Entities\Filial $filial)
    {
        return $filial->utilizaOrcamentoGOP();
    }

    public function verificaOrcamento()
    {
        try {
            if ($this->filialUtilizaOrcamentoGOP($this->notaFiscal->getFilial())) {
                $versaoMaisRecente = $this->versaoOrcamentoMaisRecente();
                $this->verificaOrcamentoDeConsumo($versaoMaisRecente);
                $this->verificaOrcamentoDeDespesas($versaoMaisRecente);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $versaoMaisRecente
     */
    protected function verificaOrcamentoDeConsumo($versaoMaisRecente)
    {
        try {
            $orcamentoParaConsumo = $this->getOrcamentoDeConsumo($versaoMaisRecente);
            if ($orcamentoParaConsumo > 0) {
                if ($this->verificaProdutoControlaEstoque() &&
                    $this->notaFiscal->getEntrsaidop()->permiteAtualizacaoDeEstoque()
                ) {
                    $volumeEntrada = $this->getVolumeEntradas();
                    if ($volumeEntrada > 0) {
                        $this->verificaPercentualAceitavelDoOrcamento($orcamentoParaConsumo, $volumeEntrada);
                        $this->verificaPercentualDeAlertaDoOrcamento($orcamentoParaConsumo, $volumeEntrada);
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $versaoMaisRecente
     */
    protected function verificaOrcamentoDeDespesas($versaoMaisRecente)
    {
        try {
            $orcamentoDeDespesas = $this->getOrcamentoDeDespesas($versaoMaisRecente);
            if ($orcamentoDeDespesas > 0) {
                if ($this->verificaProdutoControlaEstoque(Prodfili::NAO_USA_CONTROLE_ESTOQUE)) {
                    $valorDespesas = $this->getValorTotalDespesa();
                    if ($valorDespesas > 0) {
                        $this->verificaPercentualAceitavelDoOrcamento($orcamentoDeDespesas, $valorDespesas, true);
                        $this->verificaPercentualDeAlertaDoOrcamento($orcamentoDeDespesas, $valorDespesas, true);
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna a versao mais recente do Orcamento.
     *
     * @return string
     */
    protected function versaoOrcamentoMaisRecente()
    {
        try {
            $dtentrsaid           = $this->notaFiscal->getDtentrsaid();
            $nrorg                = Ambiente::getNrorg();

            /** @var $repositoryVersaoorca \Indeterminado\Repositories\Versaoorca */
            $repositoryVersaoorca = \Indeterminado\Entities\Versaoorca::getRepository();
            return $repositoryVersaoorca->obterUltimaVersaoOrcamentoDoAno($dtentrsaid, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o valor da verificacao do orcamento de consumo, obtido atraves da query em
     * \Indeterminado\Repositories\Itemorcaop::verificaOrcamentoConsumo.
     *
     * @param $orcamentoMaisRecente
     * @return float
     */
    protected function getOrcamentoDeConsumo($orcamentoMaisRecente)
    {
        try {
            /** @var $repositoryItemorcaop \Indeterminado\Repositories\Itemorcaop */
            $repositoryItemorcaop = \Contabilidade\Entities\Itemorca::getRepository();
            return $repositoryItemorcaop->getOrcamentoDeConsumo (
                $this->notaFiscal->getFilial()->getCdfilial(),
                $this->notaFiscal->getDtentrsaid(),
                $orcamentoMaisRecente,
                Ambiente::getNrorg()
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o valor de orcamento de despesas.
     *
     * @param $orcamentoMaisRecente
     * @return float
     * @throws \Exception
     */
    protected function getOrcamentoDeDespesas($orcamentoMaisRecente)
    {
        try {
            /** @var $repositoryItemorcaop \Indeterminado\Repositories\Itemorcaop */
            $repositoryItemorcaop = \Contabilidade\Entities\Itemorca::getRepository();
            return $repositoryItemorcaop->getOrcamentoDespesasEntrada(
                $this->notaFiscal->getFilial()->getCdfilial(),
                $this->notaFiscal->getDtentrsaid(),
                $orcamentoMaisRecente,
                $this->notaFiscal->getEntrsaidop()->getCdtipooper(),
                Ambiente::getNrorg()
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @return float
     * @throws \Exception
     */
    protected function getValorTotalDespesa()
    {
        try {
            /** @var $repositorioItemproduto \Geral\Repositories\Itemproduto */
            $repositorioItemproduto = \Geral\Entities\Itemproduto::getRepository();
            return $repositorioItemproduto->getVrTotalDespesa($this->notaFiscal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica se a notafiscal possui produtos com controle de estoque.
     *
     * @param string $utilizaControleEstoque
     * @throws \Exception
     * @return bool
     *
     */
    protected function verificaProdutoControlaEstoque($utilizaControleEstoque = Prodfili::USA_CONTROLE_ESTOQUE)
    {
        try {
            /** @var $repositoryItemproduto \Geral\Repositories\Itemproduto */
            $repositoryItemproduto = \Geral\Entities\Itemproduto::getRepository();
            return $repositoryItemproduto->verificaControleEstoque (
                $this->notaFiscal->getFilial()->getCdfilial(),
                $this->notaFiscal->getNrlanctonf(),
                $this->notaFiscal->getFilirece()->getCdfilial(),
                Ambiente::getNrorg(),
                $utilizaControleEstoque
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *
     *
     * @return float
     */
    protected function getVolumeEntradas()
    {
        /** @var $repositoryLanctoestoq \Estoque\Repositories\Lanctoestoq */
        $repositoryLanctoestoq = \Estoque\Entities\Lanctoestoq::getRepository();
        return $repositoryLanctoestoq->verificarVolumeEntradas (
            $this->notaFiscal->getFilirece()->getCdfilial(), $this->notaFiscal->getDtentrsaid()
        );
    }

    /**
     * Verifica se o valor de entrada da nota fiscal supera o valor aceitavel, parametrizado pela filial, para o
     * orcamento.
     *
     * @param $orcamento
     * @param $volumeEntradas
     * @param bool $orcamentoDespesas
     * @throws Exceptions\Orcamento
     */
    protected function verificaPercentualAceitavelDoOrcamento($orcamento, $volumeEntradas, $orcamentoDespesas = false)
    {
        $percentual              = $this->notaFiscal->getFilial()->getVrpeorcbloqfil();
        $valorOrcamentoAceitavel = null;

        if ($percentual > 0) {
            $valorOrcamentoAceitavel = $orcamento + ($orcamento * $percentual);
        } else {
            $valorOrcamentoAceitavel = $orcamento;
        }

        if ($volumeEntradas > $valorOrcamentoAceitavel) {
            if ($orcamentoDespesas) {
                \OrcamentoOperacional\Logic\Exceptions\Orcamento::valorDeEntradaSuperouValorDeDespesasOrcadas();
            } else {
                \OrcamentoOperacional\Logic\Exceptions\Orcamento::valorDeEntradaSuperouValorDeConsumoOrcado();
            }
        }
    }

    /**
     * Verifica se o valor de entrada da nota fiscal supera o valor de alerta parametrizado pela filial para
     * o orcamento.
     *
     * @param $orcamento
     * @param $volumeEntrada
     * @param bool $orcamentoDespesas
     */
    protected function verificaPercentualDeAlertaDoOrcamento($orcamento, $volumeEntrada, $orcamentoDespesas = false)
    {
        $valorAlerta = ($volumeEntrada * 100) / $orcamento;
        $percentual  = $this->notaFiscal->getFilial()->getVrpeorcalertfil();

        if ($valorAlerta > $percentual) {
            if ($orcamentoDespesas) {
                \OrcamentoOperacional\Logic\Exceptions\Orcamento::valorDeEntradaProximoAoValorDeConsumoOrcado();
            } else {
                \OrcamentoOperacional\Logic\Exceptions\Orcamento::valorDeEntradaProximoAoValorDeDespesasOrcadas();
            }
        }
    }

}